@@ -1,5 +1,5 @@
 /* agent.h - Global definitions for the agent
- *	Copyright (C) 2001, 2002, 2003 Free Software Foundation, Inc.
+ *	Copyright (C) 2001, 2002, 2003, 2005 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -37,7 +37,7 @@
 
 /* Convenience function to be used instead of returning the old
    GNUPG_Out_Of_Core. */
-static __inline__ gpg_error_t
+static inline gpg_error_t
 out_of_core (void)
 {
   return gpg_error (gpg_err_code_from_errno (errno));
@@ -47,25 +47,44 @@ out_of_core (void)
 
 /* A large struct name "opt" to keep global flags */
 struct {
-  unsigned int debug; /* debug flags (DBG_foo_VALUE) */
-  int verbose;      /* verbosity level */
-  int quiet;        /* be as quiet as possible */
-  int dry_run;      /* don't change any persistent data */
-  int batch;        /* batch mode */
-  const char *homedir; /* configuration directory name */
-  const char *pinentry_program; 
-  const char *scdaemon_program; 
-  int no_grab;      /* don't let the pinentry grab the keyboard */
-  unsigned long def_cache_ttl;
-  unsigned long max_cache_ttl;
+  unsigned int debug;  /* Debug flags (DBG_foo_VALUE) */
+  int verbose;         /* Verbosity level */
+  int quiet;           /* Be as quiet as possible */
+  int dry_run;         /* Don't change any persistent data */
+  int batch;           /* Batch mode */
+  const char *homedir; /* Configuration directory name */
 
-  int running_detached; /* we are running detached from the tty. */
+  /* Environment setting gathered at program start or hanged using the
+     Assuan command UPDATESTARTUPTTY. */
+  char *startup_display;
+  char *startup_ttyname;
+  char *startup_ttytype;
+  char *startup_lc_ctype;
+  char *startup_lc_messages;
+
+
+  const char *pinentry_program; /* Filename of the program to start as
+                                   pinentry.  */
+  const char *scdaemon_program; /* Filename of the program to handle
+                                   smartcard tasks.  */
+  int disable_scdaemon;         /* Never use the SCdaemon. */
+  int no_grab;         /* Don't let the pinentry grab the keyboard */
+
+  /* The default and maximum TTL of cache entries. */
+  unsigned long def_cache_ttl;     /* Default. */
+  unsigned long def_cache_ttl_ssh; /* for SSH. */
+  unsigned long max_cache_ttl;     /* Default. */
+  unsigned long max_cache_ttl_ssh; /* for SSH. */
+
+
+  int running_detached; /* We are running detached from the tty. */
 
   int ignore_cache_for_signing;
   int allow_mark_trusted;
   int allow_preset_passphrase;
-  int keep_tty;  /* don't switch the TTY (for pinentry) on request */
-  int keep_display;  /* don't switch the DISPLAY (for pinentry) on request */
+  int keep_tty;      /* Don't switch the TTY (for pinentry) on request */
+  int keep_display;  /* Don't switch the DISPLAY (for pinentry) on request */
+  int ssh_support;   /* Enable ssh-agent emulation.  */
 } opt;
 
 
@@ -86,10 +105,19 @@ struct {
 #define DBG_ASSUAN  (opt.debug & DBG_ASSUAN_VALUE)
 
 struct server_local_s;
+struct scd_local_s;
 
-struct server_control_s {
+/* Collection of data per session (aka connection). */
+struct server_control_s 
+{
+  /* Private data of the server (command.c). */
   struct server_local_s *server_local;
+
+  /* Private data of the SCdaemon (call-scd.c). */
+  struct scd_local_s *scd_local;
+
   int   connection_fd; /* -1 or an identifier for the current connection. */
+
   char *display;
   char *ttyname;
   char *ttytype;
@@ -101,9 +129,11 @@ struct server_control_s {
     int valuelen;
     int raw_value: 1;
   } digest;
-  char keygrip[20];
+  unsigned char keygrip[20];
   int have_keygrip;
 
+  int use_auth_call; /* Hack to send the PKAUTH command instead of the
+                        PKSIGN command tro scdaemon.  */
 };
 typedef struct server_control_s *CTRL;
 typedef struct server_control_s *ctrl_t;
@@ -122,20 +152,37 @@ struct pin_entry_info_s {
 };
 
 
-enum {
-  PRIVATE_KEY_UNKNOWN = 0,
-  PRIVATE_KEY_CLEAR = 1,
-  PRIVATE_KEY_PROTECTED = 2,
-  PRIVATE_KEY_SHADOWED = 3
-};
+enum 
+  {
+    PRIVATE_KEY_UNKNOWN = 0,
+    PRIVATE_KEY_CLEAR = 1,
+    PRIVATE_KEY_PROTECTED = 2,
+    PRIVATE_KEY_SHADOWED = 3
+  };
+
+
+/* Values for the cache_mode arguments. */
+typedef enum 
+  {
+    CACHE_MODE_IGNORE = 0, /* Special mode to by pass the cache. */
+    CACHE_MODE_ANY,        /* Any mode except ignore matches. */
+    CACHE_MODE_NORMAL,     /* Normal cache (gpg-agent). */
+    CACHE_MODE_USER,       /* GET_PASSPHRASE related cache. */
+    CACHE_MODE_SSH         /* SSH related cache. */
+  }
+cache_mode_t;
+
 
 /*-- gpg-agent.c --*/
-void agent_exit (int rc); /* also implemented in other tools */
+void agent_exit (int rc) JNLIB_GCC_A_NR; /* Also implemented in other tools */
 void agent_init_default_ctrl (struct server_control_s *ctrl);
 
 /*-- command.c --*/
 void start_command_handler (int, int);
 
+/*-- command-ssh.c --*/
+void start_command_handler_ssh (int);
+
 /*-- findkey.c --*/
 int agent_write_private_key (const unsigned char *grip,
                              const void *buffer, size_t length, int force);
@@ -143,13 +190,19 @@ gpg_error_t agent_key_from_file (ctrl_t ctrl,
                                  const char *desc_text,
                                  const unsigned char *grip,
                                  unsigned char **shadow_info,
-                                 int ignore_cache, gcry_sexp_t *result);
+                                 cache_mode_t cache_mode,
+                                 gcry_sexp_t *result);
+gpg_error_t agent_public_key_from_file (ctrl_t ctrl, 
+                                        const unsigned char *grip,
+                                        gcry_sexp_t *result);
 int agent_key_available (const unsigned char *grip);
 
 /*-- query.c --*/
 void initialize_module_query (void);
+void agent_query_dump_state (void);
 int agent_askpin (ctrl_t ctrl,
-                  const char *desc_text, const char *inital_errtext,
+                  const char *desc_text, const char *prompt_text,
+                  const char *inital_errtext,
                   struct pin_entry_info_s *pininfo);
 int agent_get_passphrase (ctrl_t ctrl, char **retpass,
                           const char *desc, const char *prompt,
@@ -159,16 +212,19 @@ int agent_get_confirmation (ctrl_t ctrl, const char *desc, const char *ok,
 
 /*-- cache.c --*/
 void agent_flush_cache (void);
-int agent_put_cache (const char *key, const char *data, int ttl);
-const char *agent_get_cache (const char *key, void **cache_id);
+int agent_put_cache (const char *key, cache_mode_t cache_mode,
+                     const char *data, int ttl);
+const char *agent_get_cache (const char *key, cache_mode_t cache_mode,
+                             void **cache_id);
 void agent_unlock_cache_entry (void **cache_id);
 
 
 /*-- pksign.c --*/
-int agent_pksign_do (CTRL ctrl, const char *desc_text,
-		     gcry_sexp_t *signature_sexp, int ignore_cache);
+int agent_pksign_do (ctrl_t ctrl, const char *desc_text,
+		     gcry_sexp_t *signature_sexp,
+                     cache_mode_t cache_mode);
 int agent_pksign (ctrl_t ctrl, const char *desc_text,
-                  membuf_t *outbuf, int ignore_cache);
+                  membuf_t *outbuf, cache_mode_t cache_mode);
 
 /*-- pkdecrypt.c --*/
 int agent_pkdecrypt (ctrl_t ctrl, const char *desc_text,
@@ -186,6 +242,7 @@ int agent_protect (const unsigned char *plainkey, const char *passphrase,
 int agent_unprotect (const unsigned char *protectedkey, const char *passphrase,
                      unsigned char **result, size_t *resultlen);
 int agent_private_key_type (const unsigned char *privatekey);
+unsigned char *make_shadow_info (const char *serialno, const char *idstring);
 int agent_shadow_key (const unsigned char *pubkey,
                       const unsigned char *shadow_info,
                       unsigned char **result);
@@ -216,6 +273,8 @@ int divert_generic_cmd (ctrl_t ctrl,
 
 /*-- call-scd.c --*/
 void initialize_module_call_scd (void);
+void agent_scd_dump_state (void);
+void agent_scd_check_aliveness (void);
 int agent_reset_scd (ctrl_t ctrl);
 int agent_card_learn (ctrl_t ctrl,
                       void (*kpinfo_cb)(void*, const char *),
@@ -231,7 +290,7 @@ int agent_card_pksign (ctrl_t ctrl,
                        int (*getpin_cb)(void *, const char *, char*, size_t),
                        void *getpin_cb_arg,
                        const unsigned char *indata, size_t indatalen,
-                       char **r_buf, size_t *r_buflen);
+                       unsigned char **r_buf, size_t *r_buflen);
 int agent_card_pkdecrypt (ctrl_t ctrl,
                           const char *keyid,
                           int (*getpin_cb)(void *, const char *, char*,size_t),
@@ -241,6 +300,7 @@ int agent_card_pkdecrypt (ctrl_t ctrl,
 int agent_card_readcert (ctrl_t ctrl,
                          const char *id, char **r_buf, size_t *r_buflen);
 int agent_card_readkey (ctrl_t ctrl, const char *id, unsigned char **r_buf);
+gpg_error_t agent_card_getattr (ctrl_t ctrl, const char *name, char **result);
 int agent_card_scd (ctrl_t ctrl, const char *cmdline,
                     int (*getpin_cb)(void *, const char *, char*, size_t),
                     void *getpin_cb_arg, void *assuan_context);
