@@ -1,5 +1,5 @@
 /* command.c - gpg-agent command handler
- * Copyright (C) 2001, 2002, 2003, 2004 Free Software Foundation, Inc.
+ * Copyright (C) 2001, 2002, 2003, 2004, 2005  Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -22,12 +22,14 @@
    some buffering in secure mempory to protect session keys etc. */
 
 #include <config.h>
+
 #include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
 #include <unistd.h>
+#include <assert.h>
 
 #include <assuan.h>
 
@@ -166,7 +168,7 @@ parse_keygrip (ASSUAN_CONTEXT ctx, const char *string, unsigned char *buf)
   if (n != 20)
     return set_error (Parameter_Error, "invalid length of keygrip");
 
-  for (p=string, n=0; n < 20; p += 2, n++)
+  for (p=(const unsigned char*)string, n=0; n < 20; p += 2, n++)
     buf[n] = xtoi_2 (p);
 
   return 0;
@@ -402,19 +404,19 @@ static int
 cmd_pksign (ASSUAN_CONTEXT ctx, char *line)
 {
   int rc;
-  int ignore_cache = 0;
+  cache_mode_t cache_mode = CACHE_MODE_NORMAL;
   ctrl_t ctrl = assuan_get_pointer (ctx);
   membuf_t outbuf;
-
+  
   if (opt.ignore_cache_for_signing)
-    ignore_cache = 1;
+    cache_mode = CACHE_MODE_IGNORE;
   else if (!ctrl->server_local->use_cache_for_signing)
-    ignore_cache = 1;
+    cache_mode = CACHE_MODE_IGNORE;
 
   init_membuf (&outbuf, 512);
 
   rc = agent_pksign (ctrl, ctrl->server_local->keydesc,
-                     &outbuf, ignore_cache);
+                     &outbuf, cache_mode);
   if (rc)
     clear_outbuf (&outbuf);
   else
@@ -492,7 +494,7 @@ cmd_genkey (ASSUAN_CONTEXT ctx, char *line)
 
   init_membuf (&outbuf, 512);
 
-  rc = agent_genkey (ctrl, value, valuelen, &outbuf);
+  rc = agent_genkey (ctrl, (char*)value, valuelen, &outbuf);
   xfree (value);
   if (rc)
     clear_outbuf (&outbuf);
@@ -504,6 +506,55 @@ cmd_genkey (ASSUAN_CONTEXT ctx, char *line)
 }
 
 
+
+
+/* READKEY <hexstring_with_keygrip>
+  
+   Return the public key for the given keygrip.  */
+static int
+cmd_readkey (assuan_context_t ctx, char *line)
+{
+  ctrl_t ctrl = assuan_get_pointer (ctx);
+  int rc;
+  unsigned char grip[20];
+  gcry_sexp_t s_pkey = NULL;
+
+  rc = parse_keygrip (ctx, line, grip);
+  if (rc)
+    return rc; /* Return immediately as this is already an Assuan error code.*/
+
+  rc = agent_public_key_from_file (ctrl, grip, &s_pkey);
+  if (!rc)
+    {
+      size_t len;
+      unsigned char *buf;
+
+      len = gcry_sexp_sprint (s_pkey, GCRYSEXP_FMT_CANON, NULL, 0);
+      assert (len);
+      buf = xtrymalloc (len);
+      if (!buf)
+        rc = gpg_error_from_errno (errno);
+      else
+        {
+          len = gcry_sexp_sprint (s_pkey, GCRYSEXP_FMT_CANON, buf, len);
+          assert (len);
+          rc = assuan_send_data (ctx, buf, len);
+          rc = map_assuan_err (rc);
+          xfree (buf);
+        }
+      gcry_sexp_release (s_pkey);
+    }
+
+  if (rc)
+    log_error ("command readkey failed: %s\n", gpg_strerror (rc));
+  return map_to_assuan_status (rc);
+}
+
+
+
+
+
+
 /* GET_PASSPHRASE <cache_id> [<error_message> <prompt> <description>]
 
    This function is usually used to ask for a passphrase to be used
@@ -572,7 +623,8 @@ cmd_get_passphrase (ASSUAN_CONTEXT ctx, char *line)
     desc = NULL;
 
   /* Note: we store the hexified versions in the cache. */
-  pw = cacheid ? agent_get_cache (cacheid, &cache_marker) : NULL;
+  pw = cacheid ? agent_get_cache (cacheid, CACHE_MODE_NORMAL, &cache_marker)
+               : NULL;
   if (pw)
     {
       assuan_begin_confidential (ctx);
@@ -596,7 +648,7 @@ cmd_get_passphrase (ASSUAN_CONTEXT ctx, char *line)
       if (!rc)
         {
           if (cacheid)
-            agent_put_cache (cacheid, response, 0);
+            agent_put_cache (cacheid, CACHE_MODE_USER, response, 0);
           assuan_begin_confidential (ctx);
           rc = assuan_set_okay_line (ctx, response);
           xfree (response);
@@ -631,7 +683,7 @@ cmd_clear_passphrase (ASSUAN_CONTEXT ctx, char *line)
   if (!cacheid || !*cacheid || strlen (cacheid) > 50)
     return set_error (Parameter_Error, "invalid length of cacheID");
 
-  agent_put_cache (cacheid, NULL, 0);
+  agent_put_cache (cacheid, CACHE_MODE_USER, NULL, 0);
   return 0;
 }
 
@@ -721,7 +773,7 @@ cmd_passwd (ASSUAN_CONTEXT ctx, char *line)
                   Assuan error code. */
 
   rc = agent_key_from_file (ctrl, ctrl->server_local->keydesc,
-                            grip, &shadow_info, 1, &s_skey);
+                            grip, &shadow_info, CACHE_MODE_IGNORE, &s_skey);
   if (rc)
     ;
   else if (!s_skey)
@@ -791,7 +843,7 @@ cmd_preset_passphrase (ASSUAN_CONTEXT ctx, char *line)
   else
     return map_to_assuan_status (gpg_error (GPG_ERR_NOT_IMPLEMENTED));
 
-  rc = agent_put_cache (grip_clear, passphrase, ttl);
+  rc = agent_put_cache (grip_clear, CACHE_MODE_ANY, passphrase, ttl);
 
   if (rc)
     log_error ("command preset_passwd failed: %s\n", gpg_strerror (rc));
@@ -816,6 +868,39 @@ cmd_scd (ASSUAN_CONTEXT ctx, char *line)
 }
 
 
+
+/* UPDATESTARTUPTTY 
+  
+  Set startup TTY and X DISPLAY variables to the values of this
+  session.  This command is useful to pull future pinentries to
+  another screen.  It is only required because there is no way in the
+  ssh-agent protocol to convey this information.  */
+static int
+cmd_updatestartuptty (assuan_context_t ctx, char *line)
+{
+  ctrl_t ctrl = assuan_get_pointer (ctx);
+
+  xfree (opt.startup_display); opt.startup_display = NULL;
+  xfree (opt.startup_ttyname); opt.startup_ttyname = NULL;
+  xfree (opt.startup_ttytype); opt.startup_ttytype = NULL;
+  xfree (opt.startup_lc_ctype); opt.startup_lc_ctype = NULL;
+  xfree (opt.startup_lc_messages); opt.startup_lc_messages = NULL;
+
+  if (ctrl->display)
+    opt.startup_display = xtrystrdup (ctrl->display);
+  if (ctrl->ttyname)
+    opt.startup_ttyname = xtrystrdup (ctrl->ttyname);
+  if (ctrl->ttytype)
+    opt.startup_ttytype = xtrystrdup (ctrl->ttytype);
+  if (ctrl->lc_ctype) 
+    opt.startup_lc_ctype = xtrystrdup (ctrl->lc_ctype);
+  if (ctrl->lc_messages)
+    opt.startup_lc_messages = xtrystrdup (ctrl->lc_messages);
+
+  return 0;
+}
+
+
 
 static int
 option_handler (ASSUAN_CONTEXT ctx, const char *key, const char *value)
@@ -894,6 +979,7 @@ register_commands (ASSUAN_CONTEXT ctx)
     { "PKSIGN",         cmd_pksign },
     { "PKDECRYPT",      cmd_pkdecrypt },
     { "GENKEY",         cmd_genkey },
+    { "READKEY",        cmd_readkey },
     { "GET_PASSPHRASE", cmd_get_passphrase },
     { "PRESET_PASSPHRASE", cmd_preset_passphrase },
     { "CLEAR_PASSPHRASE", cmd_clear_passphrase },
@@ -905,6 +991,7 @@ register_commands (ASSUAN_CONTEXT ctx)
     { "INPUT",          NULL }, 
     { "OUTPUT",         NULL }, 
     { "SCD",            cmd_scd },
+    { "UPDATESTARTUPTTY",  cmd_updatestartuptty },
     { NULL }
   };
   int i, rc;
@@ -1009,5 +1096,6 @@ start_command_handler (int listen_fd, int fd)
     free (ctrl.lc_ctype);
   if (ctrl.lc_messages)
     free (ctrl.lc_messages);
+  xfree (ctrl.server_local);
 }
 
