@@ -486,6 +486,7 @@ parse_bag_encrypted_data (const unsigned char *buffer, size_t length,
   buffer = p = plain;
 
 /*   { */
+/* #  warning debug code is enabled */
 /*     FILE *fp = fopen ("tmp-rc2-plain.der", "wb"); */
 /*     if (!fp || fwrite (p, n, 1, fp) != 1) */
 /*       exit (2); */
@@ -586,8 +587,10 @@ parse_bag_encrypted_data (const unsigned char *buffer, size_t length,
         }
 
       /* Ugly hack to cope with the padding: Forget about the rest if
-         that it is less than the cipher's block length. */
-      if (n < 8)
+         that is less or equal to the cipher's block length.  We can
+         reasonable assume that all valid data will be longer than
+         just one block. */
+      if (n <= 8)
         n = 0;  
 
       /* Skip the optional SET with the pkcs12 cert attributes. */
@@ -602,7 +605,7 @@ parse_bag_encrypted_data (const unsigned char *buffer, size_t length,
             { /* The optional SET. */
               p += ti.length;
               n -= ti.length;
-              if (n < 8)
+              if (n <= 8)
                 n = 0;
               if (n && parse_tag (&p, &n, &ti))
                 goto bailout;
@@ -1082,7 +1085,8 @@ create_final (struct buffer_s *sequences, size_t *r_length)
   /* Ready. */
   resultlen = p - result;
   if (needed != resultlen)
-    log_debug ("length mismatch: %u, %u\n", needed, resultlen);
+    log_debug ("length mismatch: %lu, %lu\n",
+               (unsigned long)needed, (unsigned long)resultlen);
 
   *r_length = resultlen;
   return result;
@@ -1336,7 +1340,8 @@ build_key_bag (unsigned char *buffer, size_t buflen, char *salt,
   keybaglen = p - keybag;
   
   if (needed != keybaglen)
-    log_debug ("length mismatch: %u, %u\n", needed, keybaglen);
+    log_debug ("length mismatch: %lu, %lu\n",
+               (unsigned long)needed, (unsigned long)keybaglen);
   
   *r_length = keybaglen;
   return keybag;
@@ -1428,13 +1433,14 @@ build_cert_bag (unsigned char *buffer, size_t buflen, char *salt,
   p += DIM (data_rc2iter2048);
 
   /* 8. And finally the [0] tag with the encrypted data. */
-  p = store_tag_length (p, 0xa0, buflen);
+  p = store_tag_length (p, 0x80, buflen);
   memcpy (p, buffer, buflen);
   p += buflen;
   certbaglen = p - certbag;
   
   if (needed != certbaglen)
-    log_debug ("length mismatch: %u, %u\n", needed, certbaglen);
+    log_debug ("length mismatch: %lu, %lu\n",
+               (unsigned long)needed, (unsigned long)certbaglen);
 
   *r_length = certbaglen;
   return certbag;
@@ -1524,7 +1530,8 @@ build_cert_sequence (unsigned char *buffer, size_t buflen, size_t *r_length)
   certseqlen = p - certseq;
   
   if (needed != certseqlen)
-    log_debug ("length mismatch: %u, %u\n", needed, certseqlen);
+    log_debug ("length mismatch: %lu, %lu\n",
+               (unsigned long)needed, (unsigned long)certseqlen);
   
   /* Append some pad characters; we already allocated extra space. */
   n = 8 - certseqlen % 8;
@@ -1546,9 +1553,11 @@ p12_build (gcry_mpi_t *kparms, unsigned char *cert, size_t certlen,
   unsigned char *buffer;
   size_t n, buflen;
   char salt[8];
-  struct buffer_s seqlist[2];
+  struct buffer_s seqlist[3];
   int seqlistidx = 0;
 
+  n = buflen = 0; /* (avoid compiler warning). */
+
   if (cert && certlen)
     {
       /* Encode the certificate. */
