@@ -60,6 +60,7 @@ enum cmd_and_opt_values
   oShadow,
   oShowShadowInfo,
   oShowKeygrip,
+  oCanonical,
 
   oP12Import,
   oP12Export,
@@ -86,6 +87,7 @@ struct rsa_secret_key_s
 
 static const char *opt_homedir;
 static int opt_armor;
+static int opt_canonical;
 static int opt_store;
 static int opt_force;
 static int opt_no_fail_on_exist;
@@ -107,10 +109,11 @@ static ARGPARSE_OPTS opts[] = {
 
   { oVerbose, "verbose",   0, "verbose" },
   { oArmor,   "armor",     0, "write output in advanced format" },
+  { oCanonical, "canonical", 0, "write output in canonical format" },
   { oPassphrase, "passphrase", 2, "|STRING|use passphrase STRING" },
   { oProtect, "protect",     256, "protect a private key"},
   { oUnprotect, "unprotect", 256, "unprotect a private key"},
-  { oShadow,  "shadow", 256, "create a shadow entry for a priblic key"},
+  { oShadow,  "shadow", 256, "create a shadow entry for a public key"},
   { oShowShadowInfo,  "show-shadow-info", 256, "return the shadow info"},
   { oShowKeygrip, "show-keygrip", 256, "show the \"keygrip\""},
 
@@ -236,9 +239,9 @@ make_advanced (const unsigned char *buf, size_t buflen)
   int rc;
   size_t erroff, len;
   gcry_sexp_t sexp;
-  unsigned char *result;
+  char *result;
 
-  rc = gcry_sexp_sscan (&sexp, &erroff, buf, buflen);
+  rc = gcry_sexp_sscan (&sexp, &erroff, (const char*)buf, buflen);
   if (rc)
     {
       log_error ("invalid canonical S-Expression (off=%u): %s\n",
@@ -375,7 +378,7 @@ read_and_protect (const char *fname)
       xfree (result);
       if (!p)
         return;
-      result = p;
+      result = (unsigned char*)p;
       resultlen = strlen (p);
     }
 
@@ -414,7 +417,7 @@ read_and_unprotect (const char *fname)
       xfree (result);
       if (!p)
         return;
-      result = p;
+      result = (unsigned char*)p;
       resultlen = strlen (p);
     }
 
@@ -431,12 +434,13 @@ read_and_shadow (const char *fname)
   unsigned char *key;
   unsigned char *result;
   size_t resultlen;
+  unsigned char dummy_info[] = "(8:313233342:43)";
   
   key = read_key (fname);
   if (!key)
     return;
 
-  rc = agent_shadow_key (key, "(8:313233342:43)", &result);
+  rc = agent_shadow_key (key, dummy_info, &result);
   xfree (key);
   if (rc)
     {
@@ -452,7 +456,7 @@ read_and_shadow (const char *fname)
       xfree (result);
       if (!p)
         return;
-      result = p;
+      result = (unsigned char*)p;
       resultlen = strlen (p);
     }
 
@@ -508,14 +512,21 @@ show_file (const char *fname)
 
   keylen = gcry_sexp_canon_len (key, 0, NULL,NULL);
   assert (keylen);
-
-  p = make_advanced (key, keylen);
-  xfree (key);
-  if (p)
+  
+  if (opt_canonical)
     {
-      fwrite (p, strlen (p), 1, stdout);
-      xfree (p);
+      fwrite (key, keylen, 1, stdout);
     }
+  else
+    {
+      p = make_advanced (key, keylen);
+      if (p)
+        {
+          fwrite (p, strlen (p), 1, stdout);
+          xfree (p);
+        }
+    }
+  xfree (key);
 }
 
 static void
@@ -672,7 +683,7 @@ import_p12_file (const char *fname)
   if (!buf)
     return;
 
-  kparms = p12_parse (buf, buflen, (pw=get_passphrase (2)),
+  kparms = p12_parse ((unsigned char*)buf, buflen, (pw=get_passphrase (2)),
                       import_p12_cert_cb, NULL);
   release_passphrase (pw);
   xfree (buf);
@@ -763,7 +774,7 @@ import_p12_file (const char *fname)
       xfree (result);
       if (!p)
         return;
-      result = p;
+      result = (unsigned char*)p;
       resultlen = strlen (p);
     }
 
@@ -922,7 +933,7 @@ export_p12_file (const char *fname)
 
   if (opt_have_cert)
     {
-      cert = read_file ("-", &certlen);
+      cert = (unsigned char*)read_file ("-", &certlen);
       if (!cert)
         {
           wipememory (key, keylen_for_wipe);
@@ -1030,7 +1041,7 @@ percent_plus_unescape (unsigned char *string)
 static char *
 percent_plus_unescape_string (char *string) 
 {
-  unsigned char *p = string;
+  unsigned char *p = (unsigned char*)string;
   size_t n;
 
   n = percent_plus_unescape (p);
@@ -1079,6 +1090,7 @@ main (int argc, char **argv )
         {
         case oVerbose: opt.verbose++; break;
         case oArmor:   opt_armor=1; break;
+        case oCanonical: opt_canonical=1; break;
         case oHomedir: opt_homedir = pargs.r.ret_str; break;
 
         case oProtect: cmd = oProtect; break;
