@@ -49,7 +49,7 @@
 #define LOCK_TIMEOUT  (1*60)
 
 
-static ASSUAN_CONTEXT entry_ctx = NULL;
+static assuan_context_t entry_ctx = NULL;
 #ifdef USE_GNU_PTH
 static pth_mutex_t entry_lock;
 #endif
@@ -58,7 +58,7 @@ static pth_mutex_t entry_lock;
 struct entry_parm_s {
   int lines;
   size_t size;
-  char *buffer;
+  unsigned char *buffer;
 };
 
 
@@ -82,16 +82,41 @@ initialize_module_query (void)
 
 
 
+static void
+dump_mutex_state (pth_mutex_t *m)
+{
+  if (!(m->mx_state & PTH_MUTEX_INITIALIZED))
+    log_printf ("not_initialized");
+  else if (!(m->mx_state & PTH_MUTEX_LOCKED))
+    log_printf ("not_locked");
+  else
+    log_printf ("locked tid=0x%lx count=%lu", (long)m->mx_owner, m->mx_count);
+}
+
+
+/* This function may be called to print infromation pertaining to the
+   current state of this module to the log. */
+void
+agent_query_dump_state (void)
+{
+  log_info ("agent_query_dump_state: entry_lock=");
+  dump_mutex_state (&entry_lock);
+  log_printf ("\n");
+  log_info ("agent_query_dump_state: entry_ctx=%p pid=%ld\n",
+            entry_ctx, (long)assuan_get_pid (entry_ctx));
+}
+
 
 /* Unlock the pinentry so that another thread can start one and
    disconnect that pinentry - we do this after the unlock so that a
    stalled pinentry does not block other threads.  Fixme: We should
-   have a timeout in Assuan for the disconnetc operation. */
+   have a timeout in Assuan for the disconnect operation. */
 static int 
 unlock_pinentry (int rc)
 {
-  ASSUAN_CONTEXT ctx = entry_ctx;
+  assuan_context_t ctx = entry_ctx;
 
+  entry_ctx = NULL;
 #ifdef USE_GNU_PTH
   if (!pth_mutex_release (&entry_lock))
     {
@@ -100,7 +125,6 @@ unlock_pinentry (int rc)
         rc = gpg_error (GPG_ERR_INTERNAL);
     }
 #endif
-  entry_ctx = NULL;
   assuan_disconnect (ctx);
   return rc;
 }
@@ -288,8 +312,9 @@ all_digitsp( const char *s)
    number here and repeat it as long as we have invalid formed
    numbers. */
 int
-agent_askpin (CTRL ctrl,
-              const char *desc_text, const char *initial_errtext,
+agent_askpin (ctrl_t ctrl,
+              const char *desc_text, const char *prompt_text,
+              const char *initial_errtext,
               struct pin_entry_info_s *pininfo)
 {
   int rc;
@@ -310,7 +335,10 @@ agent_askpin (CTRL ctrl,
     desc_text = _("Please enter your passphrase, so that the secret key "
                   "can be unlocked for this session");
 
-  is_pin = desc_text && strstr (desc_text, "PIN");
+  if (prompt_text)
+    is_pin = !!strstr (prompt_text, "PIN");
+  else
+    is_pin = desc_text && strstr (desc_text, "PIN");
 
   rc = start_pinentry (ctrl);
   if (rc)
@@ -322,10 +350,10 @@ agent_askpin (CTRL ctrl,
   if (rc)
     return unlock_pinentry (map_assuan_err (rc));
 
-  rc = assuan_transact (entry_ctx,
-                        is_pin? "SETPROMPT PIN:"
-                              : "SETPROMPT Passphrase:",
-                        NULL, NULL, NULL, NULL, NULL, NULL);
+  snprintf (line, DIM(line)-1, "SETPROMPT %s",
+            prompt_text? prompt_text : is_pin? "PIN:" : "Passphrase:");
+  line[DIM(line)-1] = 0;
+  rc = assuan_transact (entry_ctx, line, NULL, NULL, NULL, NULL, NULL, NULL);
   if (rc)
     return unlock_pinentry (map_assuan_err (rc));
 
@@ -344,7 +372,7 @@ agent_askpin (CTRL ctrl,
     {
       memset (&parm, 0, sizeof parm);
       parm.size = pininfo->max_length;
-      parm.buffer = pininfo->pin;
+      parm.buffer = (unsigned char*)pininfo->pin;
 
       if (errtext)
         { 
@@ -416,7 +444,8 @@ agent_get_passphrase (CTRL ctrl,
   int rc;
   char line[ASSUAN_LINELENGTH];
   struct entry_parm_s parm;
-  unsigned char *p, *hexstring;
+  unsigned char *p;
+  char *hexstring;
   int i;
 
   *retpass = NULL;
@@ -469,7 +498,7 @@ agent_get_passphrase (CTRL ctrl,
       return unlock_pinentry (map_assuan_err (rc));
     }
   
-  hexstring = gcry_malloc_secure (strlen (parm.buffer)*2+1);
+  hexstring = gcry_malloc_secure (strlen ((char*)parm.buffer)*2+1);
   if (!hexstring)
     {
       gpg_error_t tmperr = out_of_core ();
