@@ -986,7 +986,7 @@ mpi_print( FILE *fp, gcry_mpi_t a, int mode )
     }
     else {
 	int rc;
-	unsigned char *buffer;
+	char *buffer;
 
 	rc = gcry_mpi_aprint( GCRYMPI_FMT_HEX, &buffer, NULL, a );
 	assert( !rc );
