@@ -42,7 +42,6 @@
 /* A large struct name "opt" to keep global flags */
 struct {
   unsigned int debug; /* debug flags (DBG_foo_VALUE) */
-  int debug_sc;     /* OpenSC debug level */
   int verbose;      /* verbosity level */
   int quiet;        /* be as quiet as possible */
   int dry_run;      /* don't change any persistent data */
@@ -51,7 +50,6 @@ struct {
   const char *ctapi_driver; /* Library to access the ctAPI. */
   const char *pcsc_driver;  /* Library to access the PC/SC system. */
   const char *reader_port;  /* NULL or reder port to use. */
-  int disable_opensc;  /* Disable the use of the OpenSC framework. */
   int disable_ccid;    /* Disable the use of the internal CCID driver. */
   int allow_admin;     /* Allow the use of admin commands for certain
                           cards. */
@@ -79,57 +77,44 @@ struct {
 #define DBG_CARD_IO (opt.debug & DBG_CARD_IO_VALUE)
 
 struct server_local_s;
-struct card_ctx_s;
 struct app_ctx_s;
 
-struct server_control_s {
+struct server_control_s 
+{
+  /* Local data of the server; used only in command.c. */
   struct server_local_s *server_local;
-  int reader_slot;  /* Slot of the open reader or -1 if not open. */
-  struct card_ctx_s *card_ctx;
+
+  /* Slot of the open reader or -1 if not open. */
+  int reader_slot; 
+
+  /* The application context used with this connection or NULL if none
+     associated.  Note that this is shared with the other connections:
+     All connections accessing the same reader are using the same
+     application context. */
   struct app_ctx_s *app_ctx;
-  struct {
+
+  /* Helper to store the value we are going to sign */
+  struct 
+  {
     unsigned char *value;  
     int valuelen;
-  } in_data;  /* helper to store the value we are going to sign */
-
+  } in_data;  
 };
 
 typedef struct server_control_s *CTRL;
 typedef struct server_control_s *ctrl_t;
-typedef struct card_ctx_s *CARD;
 typedef struct app_ctx_s *APP;
 typedef struct app_ctx_s *app_t;
 
 /*-- scdaemon.c --*/
 void scd_exit (int rc);
-void scd_init_default_ctrl (CTRL ctrl);
+void scd_init_default_ctrl (ctrl_t ctrl);
+const char *scd_get_socket_name (void);
 
 /*-- command.c --*/
 void scd_command_handler (int);
 void send_status_info (CTRL ctrl, const char *keyword, ...);
 void scd_update_reader_status_file (void);
 
-/*-- card.c --*/
-int card_open (CARD *rcard);
-void card_close (CARD card);
-int card_get_serial_and_stamp (CARD card, char **serial, time_t *stamp);
-int card_enum_keypairs (CARD card, int idx,
-                        unsigned char *keygrip,
-                        char **keyid);
-int card_enum_certs (CARD card, int idx, char **certid, int *certtype);
-int card_read_cert (CARD card, const char *certidstr,
-                    unsigned char **cert, size_t *ncert);
-int card_sign (CARD card,
-               const char *keyidstr, int hashalgo,
-               int (pincb)(void*, const char *, char **),
-               void *pincb_arg,
-               const void *indata, size_t indatalen,
-               unsigned char **outdata, size_t *outdatalen );
-int card_decipher (CARD card, const char *keyidstr,
-                   int (pincb)(void*, const char *, char **),
-                   void *pincb_arg,
-                   const void *indata, size_t indatalen,
-                   unsigned char **outdata, size_t *outdatalen);
-
 
 #endif /*SCDAEMON_H*/
